package com.jiuqi.syntax;

package .tmp;

import com.jiuqi.syntax.CommonData;
import com.jiuqi.syntax.CommonDatas;
import com.jiuqi.syntax.DictMeaningEvent;
import com.jiuqi.syntax.FloatCopyEvent;
import com.jiuqi.syntax.FloatStatEvent;
import com.jiuqi.syntax.Function;
import com.jiuqi.syntax.FunctionListener;
import com.jiuqi.syntax.LinkPlusEvent;
import com.jiuqi.syntax.Niporlan;
import com.jiuqi.syntax.NiporlanItem;
import com.jiuqi.syntax.SyntaxEvent;
import com.jiuqi.syntax.SyntaxParser;
import com.jiuqi.syntax.UnitStateEvent;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.Stack;

public final class NiporlanParser {
  public static int NEW_COUNT = 0;
  
  private static final int ASSIGN_CELL = 0;
  
  private Stack stack = new Stack();
  
  private SyntaxParser syntaxParser;
  
  private SyntaxEvent syntaxEvent = new SyntaxEvent(this);
  
  private CommonData cellValue = new CommonData();
  
  private boolean useDataLocator;
  
  private ArrayList usedNiporlanItems;
  
  private ArrayList unUsedNiporlanItems;
  
  public NiporlanParser(SyntaxParser syntaxParser) {
    this.useDataLocator = false;
    this.usedNiporlanItems = new ArrayList();
    this.unUsedNiporlanItems = new ArrayList();
    NEW_COUNT++;
    this.syntaxParser = syntaxParser;
  }
  
  public boolean getUseDataLocator() {
    return this.useDataLocator;
  }
  
  public void setUseDataLocator(boolean value) {
    this.useDataLocator = value;
  }
  
  private NiporlanItem newNiporlanItem() {
    NiporlanItem niporlanItem;
    if (this.unUsedNiporlanItems.size() > 0) {
      niporlanItem = this.unUsedNiporlanItems.get(this.unUsedNiporlanItems.size() - 1);
      this.unUsedNiporlanItems.remove(niporlanItem);
      this.usedNiporlanItems.add(niporlanItem);
    } else {
      niporlanItem = new NiporlanItem();
      this.usedNiporlanItems.add(niporlanItem);
    } 
    return niporlanItem;
  }
  
  private void disposeNiporlanItem(NiporlanItem niporlanItem) {
    int index = this.usedNiporlanItems.indexOf(niporlanItem);
    if (index >= 0) {
      this.usedNiporlanItems.remove(index);
      this.unUsedNiporlanItems.add(niporlanItem);
    } 
  }
  
  private void disposeAllNiporlanItem() {
    this.unUsedNiporlanItems.addAll(this.usedNiporlanItems);
    this.usedNiporlanItems.clear();
  }
  
  public boolean judge(Niporlan niporlan) {
    if (niporlan.isEmpty())
      return true; 
    if (niporlan.ifExist()) {
      kernel(niporlan.ifItems, false);
      if ((((NiporlanItem)this.stack.get(0)).getCommonData()).cdBool) {
        kernel(niporlan.thenItems, false);
      } else if (niporlan.elseItems != null) {
        kernel(niporlan.elseItems, false);
      } else {
        this.stack.clear();
      } 
      if (this.stack.size() == 0)
        return true; 
      return (((NiporlanItem)this.stack.get(0)).getCommonData()).cdBool;
    } 
    kernel(niporlan.thenItems, false);
    if (this.stack.size() == 0)
      return true; 
    return (((NiporlanItem)this.stack.get(0)).getCommonData()).cdBool;
  }
  
  public void evaluate(Niporlan niporlan) {
    if (niporlan.isEmpty())
      return; 
    if (niporlan.ifExist()) {
      kernel(niporlan.ifItems, false);
      if ((((NiporlanItem)this.stack.get(0)).getCommonData()).cdBool) {
        kernel(niporlan.thenItems, true);
      } else if (niporlan.elseItems != null) {
        kernel(niporlan.elseItems, true);
      } else {
        this.stack.clear();
      } 
    } else {
      kernel(niporlan.thenItems, true);
    } 
  }
  
  public void attract(Niporlan niporlan, CommonData resultData) {
    if (niporlan.isEmpty()) {
      resultData.type = 0;
      resultData.cdReal = 0.0D;
      return;
    } 
    if (niporlan.ifExist()) {
      kernel(niporlan.ifItems, false);
      if ((((NiporlanItem)this.stack.get(0)).getCommonData()).cdBool) {
        kernel(niporlan.thenItems, false);
      } else if (niporlan.elseItems != null) {
        kernel(niporlan.elseItems, false);
      } else {
        this.stack.clear();
      } 
      if (this.stack.size() == 0) {
        resultData.type = 0;
      } else {
        resultData.assign(((NiporlanItem)this.stack.get(0)).getCommonData());
      } 
    } else {
      if (niporlan.thenItems.size() == 1) {
        NiporlanItem ni = niporlan.thenItems.get(0);
        if (ni.getType() == 3) {
          this.syntaxEvent.dataCell = ni.getDataCell();
          this.syntaxEvent.value = this.cellValue;
          if (this.useDataLocator) {
            getCellValue(this.syntaxEvent, ni);
          } else {
            getCellValue(this.syntaxEvent);
          } 
          resultData.assign(this.syntaxEvent.value);
          if (resultData.type == 1) {
            resultData.type = 2;
            resultData.cdReal = resultData.cdInt;
          } 
          if (resultData.type == 0)
            error("Niporlan公式有错误(空数据类型)！"); 
          return;
        } 
      } 
      kernel(niporlan.thenItems, false);
      if (this.stack.size() == 0) {
        resultData.type = 0;
      } else {
        resultData.assign(((NiporlanItem)this.stack.get(0)).getCommonData());
      } 
    } 
  }
  
  private void bool2Real(CommonData data) {
    if (data.type == 5) {
      data.type = 2;
      if (data.cdBool) {
        data.cdReal = -1.0D;
      } else {
        data.cdReal = 0.0D;
      } 
    } 
  }
  
  private void real2Decimal(CommonData data) {
    if (data.type == 2) {
      data.type = 10;
      data.cdDecimal = new BigDecimal(data.cdReal);
    } 
  }
  
  private void kernel(ArrayList<NiporlanItem> niItems, boolean isAssignValue) {
    int assignKind = 0;
    this.stack.clear();
    disposeAllNiporlanItem();
    for (int i = 0, n = niItems.size(); i < n; i++) {
      NiporlanItem rightItem, leftItem;
      CommonData leftItemCommonData, rightItemCommonData;
      FunctionListener funcListener;
      Function func;
      NiporlanItem[] niporlanItems;
      CommonDatas paramValues;
      int j;
      NiporlanItem item = niItems.get(i);
      switch (item.getType()) {
        case 0:
          switch (item.getOp()) {
            case 1:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              leftItemCommonData = leftItem.getCommonData();
              rightItemCommonData = rightItem.getCommonData();
              bool2Real(rightItemCommonData);
              bool2Real(leftItemCommonData);
              if (rightItemCommonData.type == 10)
                real2Decimal(leftItemCommonData); 
              switch (leftItemCommonData.type) {
                case 2:
                  leftItemCommonData.cdReal += rightItemCommonData.cdReal;
                  break;
                case 3:
                  leftItemCommonData.cdString = String.valueOf(leftItemCommonData.cdString) + rightItemCommonData.cdString;
                  break;
                case 10:
                  real2Decimal(rightItemCommonData);
                  if (leftItemCommonData.isNull) {
                    leftItemCommonData.cdDecimal = rightItemCommonData.cdDecimal;
                    break;
                  } 
                  if (!rightItemCommonData.isNull)
                    leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.add(rightItemCommonData.cdDecimal); 
                  break;
              } 
              leftItemCommonData.isNull = (leftItemCommonData.isNull && rightItemCommonData.isNull);
              leftItemCommonData.initNullValue();
              disposeNiporlanItem(rightItem);
              break;
            case 2:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              leftItemCommonData = leftItem.getCommonData();
              rightItemCommonData = rightItem.getCommonData();
              bool2Real(rightItemCommonData);
              bool2Real(leftItemCommonData);
              if (rightItemCommonData.type == 10)
                real2Decimal(leftItemCommonData); 
              switch (leftItemCommonData.type) {
                case 2:
                  leftItemCommonData.cdReal -= rightItemCommonData.cdReal;
                  if (Math.abs(leftItemCommonData.cdReal) < 1.0E-5D)
                    leftItemCommonData.cdReal = 0.0D; 
                  break;
                case 6:
                  leftItemCommonData.type = 2;
                  leftItemCommonData.cdReal = SyntaxParser.minusDate(leftItemCommonData.cdDate, rightItemCommonData.cdDate);
                  if (Math.abs(leftItemCommonData.cdReal) < 1.0E-5D)
                    leftItemCommonData.cdReal = 0.0D; 
                  break;
                case 10:
                  real2Decimal(rightItemCommonData);
                  if (leftItemCommonData.isNull) {
                    leftItemCommonData.cdDecimal = rightItemCommonData.cdDecimal.negate();
                    break;
                  } 
                  if (!rightItemCommonData.isNull)
                    leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.subtract(rightItemCommonData.cdDecimal); 
                  break;
              } 
              leftItemCommonData.isNull = (leftItemCommonData.isNull && rightItemCommonData.isNull);
              leftItemCommonData.initNullValue();
              disposeNiporlanItem(rightItem);
              break;
            case 3:
              leftItem = this.stack.peek();
              leftItemCommonData = leftItem.getCommonData();
              bool2Real(leftItemCommonData);
              if (leftItemCommonData.type == 10) {
                if (!leftItemCommonData.isNull)
                  leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.negate(); 
              } else {
                (leftItem.getCommonData()).cdReal = -(leftItem.getCommonData()).cdReal;
              } 
              leftItem.getCommonData().initNullValue();
              break;
            case 4:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              leftItemCommonData = leftItem.getCommonData();
              rightItemCommonData = rightItem.getCommonData();
              bool2Real(rightItemCommonData);
              bool2Real(leftItemCommonData);
              if (rightItemCommonData.type == 10)
                real2Decimal(leftItemCommonData); 
              if (leftItemCommonData.type == 10) {
                if (!leftItemCommonData.isNull && !rightItemCommonData.isNull) {
                  real2Decimal(rightItemCommonData);
                  leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.multiply(rightItemCommonData.cdDecimal);
                } 
              } else {
                leftItemCommonData.cdReal *= rightItemCommonData.cdReal;
              } 
              leftItemCommonData.isNull = !(!leftItemCommonData.isNull && !rightItemCommonData.isNull);
              leftItemCommonData.initNullValue();
              disposeNiporlanItem(rightItem);
              break;
            case 5:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              leftItemCommonData = leftItem.getCommonData();
              rightItemCommonData = rightItem.getCommonData();
              bool2Real(rightItemCommonData);
              bool2Real(leftItemCommonData);
              if (rightItemCommonData.type == 10)
                real2Decimal(leftItemCommonData); 
              if (leftItemCommonData.type == 10) {
                if (!leftItemCommonData.isNull && !rightItemCommonData.isNull) {
                  real2Decimal(rightItemCommonData);
                  if (rightItemCommonData.cdDecimal.abs().compareTo(new BigDecimal(1.0E-5D)) < 0) {
                    leftItemCommonData.cdDecimal = new BigDecimal(0);
                  } else {
                    leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal
                      .divide(rightItemCommonData.cdDecimal, 
                        4);
                  } 
                } 
              } else if (Math.abs(rightItemCommonData.cdReal) < 1.0E-5D) {
                leftItemCommonData.cdReal = 0.0D;
              } else {
                leftItemCommonData.cdReal /= rightItemCommonData.cdReal;
              } 
              leftItemCommonData.isNull = !(!leftItemCommonData.isNull && !rightItemCommonData.isNull);
              leftItemCommonData.initNullValue();
              disposeNiporlanItem(rightItem);
              break;
            case 7:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              leftItemCommonData = leftItem.getCommonData();
              rightItemCommonData = rightItem.getCommonData();
              if (rightItemCommonData.type == 10)
                real2Decimal(leftItemCommonData); 
              if (leftItemCommonData.type == 10)
                real2Decimal(rightItemCommonData); 
              leftItemCommonData.cdBool = leftItemCommonData.equals(rightItemCommonData, this.syntaxParser.getBalanceDifferenceValue());
              leftItemCommonData.type = 5;
              leftItemCommonData.isNull = false;
              disposeNiporlanItem(rightItem);
              break;
            case 8:
              break;
            case 9:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              leftItemCommonData = leftItem.getCommonData();
              rightItemCommonData = rightItem.getCommonData();
              if (rightItemCommonData.type == 10)
                real2Decimal(leftItemCommonData); 
              if (leftItemCommonData.type == 10)
                real2Decimal(rightItemCommonData); 
              (leftItem.getCommonData()).cdBool = !leftItem.getCommonData().equals(rightItem.getCommonData(), this.syntaxParser.getBalanceDifferenceValue());
              (leftItem.getCommonData()).type = 5;
              (leftItem.getCommonData()).isNull = false;
              disposeNiporlanItem(rightItem);
              break;
            case 10:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              leftItemCommonData = leftItem.getCommonData();
              rightItemCommonData = rightItem.getCommonData();
              if (rightItemCommonData.type == 10)
                real2Decimal(leftItemCommonData); 
              if (leftItemCommonData.type == 10)
                real2Decimal(rightItemCommonData); 
              (leftItem.getCommonData()).cdBool = (leftItem.getCommonData().compareTo(rightItem.getCommonData(), 1.0E-5D) > 0);
              (leftItem.getCommonData()).type = 5;
              (leftItem.getCommonData()).isNull = false;
              disposeNiporlanItem(rightItem);
              break;
            case 11:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              leftItemCommonData = leftItem.getCommonData();
              rightItemCommonData = rightItem.getCommonData();
              if (rightItemCommonData.type == 10)
                real2Decimal(leftItemCommonData); 
              if (leftItemCommonData.type == 10)
                real2Decimal(rightItemCommonData); 
              (leftItem.getCommonData()).cdBool = (leftItem.getCommonData().compareTo(rightItem.getCommonData(), 1.0E-5D) < 0);
              (leftItem.getCommonData()).type = 5;
              (leftItem.getCommonData()).isNull = false;
              disposeNiporlanItem(rightItem);
              break;
            case 12:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              leftItemCommonData = leftItem.getCommonData();
              rightItemCommonData = rightItem.getCommonData();
              if (rightItemCommonData.type == 10)
                real2Decimal(leftItemCommonData); 
              if (leftItemCommonData.type == 10)
                real2Decimal(rightItemCommonData); 
              (leftItem.getCommonData()).cdBool = (leftItem.getCommonData().compareTo(rightItem.getCommonData(), this.syntaxParser.getBalanceDifferenceValue()) >= 0);
              (leftItem.getCommonData()).type = 5;
              (leftItem.getCommonData()).isNull = false;
              disposeNiporlanItem(rightItem);
              break;
            case 13:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              leftItemCommonData = leftItem.getCommonData();
              rightItemCommonData = rightItem.getCommonData();
              if (rightItemCommonData.type == 10)
                real2Decimal(leftItemCommonData); 
              if (leftItemCommonData.type == 10)
                real2Decimal(rightItemCommonData); 
              (leftItem.getCommonData()).cdBool = (leftItem.getCommonData().compareTo(rightItem.getCommonData(), this.syntaxParser.getBalanceDifferenceValue()) <= 0);
              (leftItem.getCommonData()).type = 5;
              (leftItem.getCommonData()).isNull = false;
              disposeNiporlanItem(rightItem);
              break;
            case 14:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              (leftItem.getCommonData()).cdBool = ((leftItem.getCommonData()).cdBool && (rightItem.getCommonData()).cdBool);
              (leftItem.getCommonData()).isNull = ((leftItem.getCommonData()).isNull && (rightItem.getCommonData()).isNull);
              leftItem.getCommonData().initNullValue();
              disposeNiporlanItem(rightItem);
              break;
            case 15:
              rightItem = this.stack.pop();
              leftItem = this.stack.peek();
              (leftItem.getCommonData()).cdBool = !(!(leftItem.getCommonData()).cdBool && !(rightItem.getCommonData()).cdBool);
              (leftItem.getCommonData()).isNull = ((leftItem.getCommonData()).isNull && (rightItem.getCommonData()).isNull);
              leftItem.getCommonData().initNullValue();
              disposeNiporlanItem(rightItem);
              break;
            case 16:
              leftItem = this.stack.peek();
              (leftItem.getCommonData()).cdBool = !(leftItem.getCommonData()).cdBool;
              leftItem.getCommonData().initNullValue();
              break;
            case 17:
              rightItem = this.stack.pop();
              leftItem = this.stack.pop();
              if (assignKind == 0) {
                this.syntaxEvent.dataCell = leftItem.getDataCell();
                this.syntaxEvent.value = rightItem.getCommonData();
                if (this.useDataLocator) {
                  setCellValue(this.syntaxEvent, leftItem.getDataLocator());
                } else {
                  setCellValue(this.syntaxEvent);
                } 
              } 
              disposeNiporlanItem(rightItem);
              disposeNiporlanItem(leftItem);
              break;
          } 
          error();
          break;
        case 1:
          if (item.getFuncName().equals("SUM")) {
            leftItem = this.stack.peek();
            this.syntaxEvent.dataCell = leftItem.getDataCell();
            this.syntaxEvent.value = this.cellValue;
            getStatValue(this.syntaxEvent);
            leftItem.setType(2);
            leftItem.getCommonData().assign(this.syntaxEvent.value);
            if ((leftItem.getCommonData()).type == 1) {
              (leftItem.getCommonData()).type = 2;
              (leftItem.getCommonData()).cdReal = (leftItem.getCommonData()).cdInt;
            } 
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("L$")) {
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdString = SyntaxParser.asciiCopy((leftItem.getCommonData()).cdString, 1, (int)(rightItem.getCommonData()).cdReal);
            leftItem.getCommonData().initNullValue();
            disposeNiporlanItem(rightItem);
            break;
          } 
          if (item.getFuncName().equals("R$")) {
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            if (SyntaxParser.asciiLength((leftItem.getCommonData()).cdString) + 1 - (int)(rightItem.getCommonData()).cdReal >= 1) {
              (leftItem.getCommonData()).cdString = SyntaxParser.asciiCopy((leftItem.getCommonData()).cdString, SyntaxParser.asciiLength((leftItem.getCommonData()).cdString) + 1 - (int)(rightItem.getCommonData()).cdReal, (int)(rightItem.getCommonData()).cdReal);
            } else {
              (leftItem.getCommonData()).cdString = (leftItem.getCommonData()).cdString;
            } 
            leftItem.getCommonData().initNullValue();
            disposeNiporlanItem(rightItem);
            break;
          } 
          if (item.getFuncName().equals("M$")) {
            NiporlanItem threeItem = this.stack.pop();
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdString = SyntaxParser.asciiCopy((leftItem.getCommonData()).cdString, (int)(rightItem.getCommonData()).cdReal, (int)(threeItem.getCommonData()).cdReal);
            leftItem.getCommonData().initNullValue();
            disposeNiporlanItem(rightItem);
            disposeNiporlanItem(threeItem);
            break;
          } 
          if (item.getFuncName().equals("POS")) {
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = SyntaxParser.asciiPos((leftItem.getCommonData()).cdString, (rightItem.getCommonData()).cdString);
            (leftItem.getCommonData()).type = 2;
            (leftItem.getCommonData()).isNull = false;
            disposeNiporlanItem(rightItem);
            break;
          } 
          if (item.getFuncName().equals("ABS")) {
            leftItem = this.stack.peek();
            leftItemCommonData = leftItem.getCommonData();
            if (leftItemCommonData.type == 10) {
              if (!leftItemCommonData.isNull)
                leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.abs(); 
            } else {
              leftItemCommonData.cdReal = Math.abs((leftItem.getCommonData()).cdReal);
            } 
            leftItemCommonData.initNullValue();
            break;
          } 
          if (item.getFuncName().equals("INT")) {
            leftItem = this.stack.peek();
            leftItemCommonData = leftItem.getCommonData();
            if (leftItemCommonData.type == 10) {
              if (!leftItemCommonData.isNull) {
                leftItemCommonData.cdReal = leftItemCommonData.cdDecimal.intValue();
                leftItemCommonData.type = 2;
                leftItemCommonData.cdDecimal = null;
              } 
            } else {
              leftItemCommonData.cdReal = SyntaxParser.trunc((leftItem.getCommonData()).cdReal);
            } 
            leftItemCommonData.initNullValue();
            break;
          } 
          if (item.getFuncName().equals("ROUND")) {
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            leftItemCommonData = leftItem.getCommonData();
            if (leftItemCommonData.type == 10) {
              if (!leftItemCommonData.isNull)
                leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal
                  .setScale(
                    (int)(rightItem.getCommonData()).cdReal, 
                    4); 
            } else {
              (leftItem.getCommonData()).cdReal = SyntaxParser.roundFloat((leftItem.getCommonData()).cdReal, (int)(rightItem.getCommonData()).cdReal);
            } 
            leftItem.getCommonData().initNullValue();
            disposeNiporlanItem(rightItem);
            break;
          } 
          if (item.getFuncName().equals("MOD")) {
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            leftItemCommonData = leftItem.getCommonData();
            if (leftItemCommonData.type == 10) {
              if (!leftItemCommonData.isNull) {
                leftItemCommonData.cdReal = SyntaxParser.floatMod(
                    leftItemCommonData.cdDecimal.doubleValue(), 
                    (rightItem.getCommonData()).cdReal);
                leftItemCommonData.type = 2;
              } 
            } else {
              (leftItem.getCommonData()).cdReal = SyntaxParser.floatMod((leftItem.getCommonData()).cdReal, (rightItem.getCommonData()).cdReal);
            } 
            (leftItem.getCommonData()).isNull = !(!(leftItem.getCommonData()).isNull && !(rightItem.getCommonData()).isNull);
            leftItem.getCommonData().initNullValue();
            disposeNiporlanItem(rightItem);
            break;
          } 
          if (item.getFuncName().equals("POWER")) {
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = Math.pow((leftItem.getCommonData()).cdReal, (rightItem.getCommonData()).cdReal);
            (leftItem.getCommonData()).isNull = !(!(leftItem.getCommonData()).isNull && !(rightItem.getCommonData()).isNull);
            leftItem.getCommonData().initNullValue();
            disposeNiporlanItem(rightItem);
            break;
          } 
          if (item.getFuncName().equals("TAN")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = Math.tan((leftItem.getCommonData()).cdReal);
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("ATAN")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = Math.atan((leftItem.getCommonData()).cdReal);
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("LN")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = Math.log((leftItem.getCommonData()).cdReal);
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("LOG10")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = Math.log((leftItem.getCommonData()).cdReal) / Math.log(10.0D);
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("LOG")) {
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = Math.log((leftItem.getCommonData()).cdReal) / Math.log((rightItem.getCommonData()).cdReal);
            (leftItem.getCommonData()).isNull = !(!(leftItem.getCommonData()).isNull && !(rightItem.getCommonData()).isNull);
            leftItem.getCommonData().initNullValue();
            disposeNiporlanItem(rightItem);
            break;
          } 
          if (item.getFuncName().equals("VAL")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = SyntaxParser.safeStrToFloat((leftItem.getCommonData()).cdString);
            (leftItem.getCommonData()).type = 2;
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("STR")) {
            NiporlanItem threeItem = this.stack.pop();
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            if ((leftItem.getCommonData()).type == 1 || (leftItem.getCommonData()).type == 2) {
              (leftItem.getCommonData()).cdString = SyntaxParser.FloatToStr((leftItem.getCommonData()).cdReal, (int)(rightItem.getCommonData()).cdReal, (threeItem.getCommonData()).cdBool);
            } else if ((leftItem.getCommonData()).type == 6) {
              (leftItem.getCommonData()).cdString = ((leftItem.getCommonData()).cdDate == null) ? "" : SyntaxParser.datetimeToStr((leftItem.getCommonData()).cdDate);
            } else {
              (leftItem.getCommonData()).cdString = leftItem.getCommonData().toString();
            } 
            (leftItem.getCommonData()).type = 3;
            leftItem.getCommonData().initNullValue();
            disposeNiporlanItem(rightItem);
            disposeNiporlanItem(threeItem);
            break;
          } 
          if (item.getFuncName().equals("IDC")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdString = String.valueOf(SyntaxParser.idc((leftItem.getCommonData()).cdString));
            (leftItem.getCommonData()).isNull = false;
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("ISDIGIT")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdBool = SyntaxParser.isDigit((leftItem.getCommonData()).cdString);
            (leftItem.getCommonData()).type = 5;
            (leftItem.getCommonData()).isNull = false;
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("LEN")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = SyntaxParser.asciiLength((leftItem.getCommonData()).cdString);
            (leftItem.getCommonData()).type = 2;
            (leftItem.getCommonData()).isNull = false;
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("ISNULL")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdBool = (leftItem.getCommonData()).isNull;
            (leftItem.getCommonData()).type = 5;
            (leftItem.getCommonData()).isNull = false;
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("EXISTCHINESE")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdBool = SyntaxParser.existChinese((leftItem.getCommonData()).cdString);
            (leftItem.getCommonData()).type = 5;
            (leftItem.getCommonData()).isNull = false;
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("INLIST")) {
            leftItem = this.stack.pop();
            int count = (int)(leftItem.getCommonData()).cdReal;
            (leftItem.getCommonData()).type = 5;
            (leftItem.getCommonData()).cdBool = false;
            String[] list = new String[count];
            int l;
            for (l = 0; l < count; l++) {
              rightItem = this.stack.pop();
              list[l] = (rightItem.getCommonData()).cdString;
              disposeNiporlanItem(rightItem);
            } 
            NiporlanItem threeItem = this.stack.pop();
            for (l = 0; l < count; l++) {
              if (list[l].equals((threeItem.getCommonData()).cdString) || (!list[l].equals("") && (threeItem.getCommonData()).cdString.indexOf(list[l]) == 0)) {
                (leftItem.getCommonData()).cdBool = true;
                break;
              } 
            } 
            disposeNiporlanItem(threeItem);
            (leftItem.getCommonData()).isNull = false;
            leftItem.getCommonData().initNullValue();
            this.stack.push(leftItem);
            break;
          } 
          if (item.getFuncName().equals("INDICT")) {
            leftItem = this.stack.pop();
            rightItem = this.stack.peek();
            DictMeaningEvent event = new DictMeaningEvent(this);
            event.code = (rightItem.getCommonData()).cdString;
            event.dictName = (leftItem.getCommonData()).cdString;
            getDictMeaning(event);
            (rightItem.getCommonData()).type = 5;
            (rightItem.getCommonData()).cdBool = event.inDict;
            (rightItem.getCommonData()).isNull = false;
            rightItem.getCommonData().initNullValue();
            disposeNiporlanItem(leftItem);
            break;
          } 
          if (item.getFuncName().equals("DICTVALUE")) {
            NiporlanItem threeItem = this.stack.pop();
            leftItem = this.stack.pop();
            rightItem = this.stack.peek();
            DictMeaningEvent event = new DictMeaningEvent(this);
            event.code = (rightItem.getCommonData()).cdString;
            event.dictName = (leftItem.getCommonData()).cdString;
            event.meaningName = (threeItem.getCommonData()).cdString;
            getDictMeaning(event);
            rightItem.getCommonData().assign(event.meaningValue);
            if ((rightItem.getCommonData()).type == 1) {
              (rightItem.getCommonData()).type = 2;
              (rightItem.getCommonData()).cdReal = (rightItem.getCommonData()).cdInt;
            } 
            rightItem.getCommonData().initNullValue();
            disposeNiporlanItem(leftItem);
            disposeNiporlanItem(threeItem);
            break;
          } 
          if (item.getFuncName().equals("UNITSTATE")) {
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            UnitStateEvent event = new UnitStateEvent(this, (leftItem.getCommonData()).cdString, (rightItem.getCommonData()).cdString);
            getUnitState(event);
            (leftItem.getCommonData()).isNull = false;
            (leftItem.getCommonData()).type = 2;
            (leftItem.getCommonData()).cdReal = event.state;
            disposeNiporlanItem(rightItem);
            break;
          } 
          if (item.getFuncName().equals("LINKPLUS")) {
            NiporlanItem threeItem = this.stack.pop();
            leftItem = this.stack.pop();
            rightItem = this.stack.peek();
            LinkPlusEvent event = new LinkPlusEvent(this, rightItem.getDataCell(), (int)(leftItem.getCommonData()).cdReal, (int)(threeItem.getCommonData()).cdReal, this.cellValue);
            getLinkPlus(event);
            rightItem.getCommonData().assign(event.value);
            if ((rightItem.getCommonData()).type == 1) {
              (rightItem.getCommonData()).type = 2;
              (rightItem.getCommonData()).cdReal = (rightItem.getCommonData()).cdInt;
            } 
            rightItem.getCommonData().initNullValue();
            disposeNiporlanItem(leftItem);
            disposeNiporlanItem(threeItem);
            break;
          } 
          if (item.getFuncName().equals("FLOATSTAT")) {
            leftItem = this.stack.peek();
            FloatStatEvent event = (FloatStatEvent)leftItem.getObject();
            getFloatStat(event);
            leftItem.setType(2);
            leftItem.getCommonData().assign(event.value);
            if ((leftItem.getCommonData()).type == 1) {
              (leftItem.getCommonData()).type = 2;
              (leftItem.getCommonData()).cdReal = (leftItem.getCommonData()).cdInt;
            } 
            leftItem.getCommonData().initNullValue();
            leftItem.setObject(null);
            break;
          } 
          if (item.getFuncName().equals("FLOATCOPY")) {
            leftItem = this.stack.peek();
            FloatCopyEvent event = (FloatCopyEvent)leftItem.getObject();
            execFloatCopy(event);
            leftItem.setType(2);
            (leftItem.getCommonData()).isNull = false;
            (leftItem.getCommonData()).type = 2;
            (leftItem.getCommonData()).cdReal = event.copyRowCount;
            leftItem.setObject(null);
            break;
          } 
          if (item.getFuncName().equals("DATEVALUE")) {
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).isNull = !SyntaxParser.safeStrToDate(leftItem.getCommonData().getValidcdDate(), (leftItem.getCommonData()).cdString, (rightItem.getCommonData()).cdString);
            (leftItem.getCommonData()).type = 6;
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("FORMATDATE")) {
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdString = SyntaxParser.formatDate(leftItem.getCommonData().getValidcdDate(), (rightItem.getCommonData()).cdString);
            (leftItem.getCommonData()).type = 3;
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("DATE")) {
            NiporlanItem threeItem = this.stack.pop();
            rightItem = this.stack.pop();
            leftItem = this.stack.peek();
            SyntaxParser.safeEncodeDate(leftItem.getCommonData().getValidcdDate(), (int)(leftItem.getCommonData()).cdReal, (int)(rightItem.getCommonData()).cdReal, (int)(threeItem.getCommonData()).cdReal);
            (leftItem.getCommonData()).type = 6;
            (leftItem.getCommonData()).isNull = !(!(leftItem.getCommonData()).isNull && !(rightItem.getCommonData()).isNull && !(threeItem.getCommonData()).isNull);
            leftItem.getCommonData().initNullValue();
            disposeNiporlanItem(rightItem);
            disposeNiporlanItem(threeItem);
            break;
          } 
          if (item.getFuncName().equals("YEAR")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = leftItem.getCommonData().getValidcdDate().get(1);
            (leftItem.getCommonData()).type = 2;
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("MONTH")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = (leftItem.getCommonData().getValidcdDate().get(2) + 1);
            (leftItem.getCommonData()).type = 2;
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("DAY")) {
            leftItem = this.stack.peek();
            (leftItem.getCommonData()).cdReal = leftItem.getCommonData().getValidcdDate().get(5);
            (leftItem.getCommonData()).type = 2;
            leftItem.getCommonData().initNullValue();
            break;
          } 
          if (item.getFuncName().equals("TODAY")) {
            leftItem = newNiporlanItem();
            leftItem.setValue(item.getCommonData());
            (leftItem.getCommonData()).type = 6;
            (leftItem.getCommonData()).cdDate = new GregorianCalendar();
            (leftItem.getCommonData()).cdDate.getTime().setTime(0L);
            this.stack.push(leftItem);
            break;
          } 
          funcListener = this.syntaxParser.getFunctionListener(item.getFuncName());
          if (funcListener == null) {
            error("没有找到函数“" + item.getFuncName() + "”的侦听器！");
            return;
          } 
          func = funcListener.funcByName(item.getFuncName());
          paramValues = new CommonDatas();
          if (!func.isInfiniteParam) {
            niporlanItems = new NiporlanItem[func.params.size()];
            for (int k = 0; k < func.params.size(); k++)
              niporlanItems[func.params.size() - k - 1] = this.stack.pop(); 
          } else {
            leftItem = this.stack.pop();
            niporlanItems = new NiporlanItem[(leftItem.getCommonData()).cdInt];
            for (int k = 0; k < (leftItem.getCommonData()).cdInt; k++)
              niporlanItems[(leftItem.getCommonData()).cdInt - k - 1] = this.stack.pop(); 
            disposeNiporlanItem(leftItem);
          } 
          for (j = 0; j < niporlanItems.length; j++)
            paramValues.add(niporlanItems[j].getCommonData()); 
          leftItem = newNiporlanItem();
          leftItem.setType(2);
          (leftItem.getCommonData()).type = func.resultType;
          (leftItem.getCommonData()).isNull = false;
          leftItem.getCommonData().initValue();
          if (funcListener.callFunction(func.name, paramValues, leftItem.getCommonData(), false) != 0)
            leftItem.setValue(leftItem.getCommonData()); 
          if ((leftItem.getCommonData()).type == 1) {
            (leftItem.getCommonData()).type = 2;
            (leftItem.getCommonData()).cdReal = (leftItem.getCommonData()).cdInt;
          } 
          leftItem.getCommonData().initNullValue();
          this.stack.push(leftItem);
          break;
        case 2:
          leftItem = newNiporlanItem();
          leftItem.setValue(item.getCommonData());
          leftItemCommonData = leftItem.getCommonData();
          if (leftItemCommonData.type == 1) {
            leftItemCommonData.type = 2;
            leftItemCommonData.cdReal = leftItemCommonData.cdInt;
          } 
          this.stack.push(leftItem);
          break;
        case 3:
          leftItem = newNiporlanItem();
          leftItem.setValue(item.getDataCell());
          if (this.useDataLocator)
            leftItem.setDataLocator(item); 
          if (isAssignValue && this.stack.size() == 0) {
            assignKind = 0;
          } else {
            this.syntaxEvent.dataCell = leftItem.getDataCell();
            this.syntaxEvent.value = this.cellValue;
            if (this.useDataLocator) {
              getCellValue(this.syntaxEvent, leftItem.getDataLocator());
            } else {
              getCellValue(this.syntaxEvent);
            } 
            leftItem.setType(2);
            leftItemCommonData = leftItem.getCommonData();
            leftItemCommonData.assign(this.syntaxEvent.value);
            if (leftItemCommonData.type == 1) {
              leftItemCommonData.type = 2;
              leftItemCommonData.cdReal = leftItemCommonData.cdInt;
            } 
            if (leftItemCommonData.type == 0)
              error("Niporlan公式有错误(空数据类型)！"); 
            leftItemCommonData.initNullValue();
          } 
          this.stack.push(leftItem);
          break;
        case 4:
          leftItem = newNiporlanItem();
          leftItem.setValue(item.getObject());
          this.stack.push(leftItem);
          break;
        default:
          error();
          break;
      } 
    } 
  }
  
  private void getCellValue(SyntaxEvent syntaxEvent) {
    if (this.syntaxParser.getSyntaxListener() != null)
      this.syntaxParser.getSyntaxListener().getNiValue(syntaxEvent); 
  }
  
  private void getCellValue(SyntaxEvent syntaxEvent, Object dataLocator) {
    if (this.syntaxParser.getOptimizeListener() != null)
      this.syntaxParser.getOptimizeListener().getCellValue(syntaxEvent, dataLocator); 
  }
  
  private void setCellValue(SyntaxEvent syntaxEvent) {
    if (this.syntaxParser.getSyntaxListener() != null)
      this.syntaxParser.getSyntaxListener().setNiValue(syntaxEvent); 
  }
  
  private void setCellValue(SyntaxEvent syntaxEvent, Object dataLocator) {
    if (this.syntaxParser.getOptimizeListener() != null)
      this.syntaxParser.getOptimizeListener().setCellValue(syntaxEvent, dataLocator); 
  }
  
  private void getStatValue(SyntaxEvent syntaxEvent) {
    if (this.syntaxParser.getSyntaxListener() != null)
      this.syntaxParser.getSyntaxListener().getStatValue(syntaxEvent); 
  }
  
  private void getUnitState(UnitStateEvent unitStateEvent) {
    if (this.syntaxParser.getSyntaxListener() != null)
      this.syntaxParser.getSyntaxListener().getUnitState(unitStateEvent); 
  }
  
  private void getLinkPlus(LinkPlusEvent linkPlusEvent) {
    if (this.syntaxParser.getSyntaxListener() != null)
      this.syntaxParser.getSyntaxListener().getLinkPlus(linkPlusEvent); 
  }
  
  private void getFloatStat(FloatStatEvent floatStatEvent) {
    if (this.syntaxParser.getSyntaxListener() != null)
      this.syntaxParser.getSyntaxListener().getFloatStat(floatStatEvent); 
  }
  
  private void execFloatCopy(FloatCopyEvent floatCopyEvent) {
    if (this.syntaxParser.getSyntaxListener() != null)
      this.syntaxParser.getSyntaxListener().execFloatCopy(floatCopyEvent); 
  }
  
  private void getDictMeaning(DictMeaningEvent dictMeaningEvent) {
    if (this.syntaxParser.getSyntaxListener() != null)
      this.syntaxParser.getSyntaxListener().getDictMeaning(dictMeaningEvent); 
  }
  
  private void error() {
    error("Niporlan公式有错误 或 不支持的操作！");
  }
  
  private void error(String message) {
    System.out.println(message);
  }
}
